;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
; Headerdatei
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;Der ATmega88PA laeuft mit einem 1,8432 MHz Quarz und wird an 3,3V betrieben.
;
;FUSES Extendet $F9, High $97, Low $EA
;debugWIRE Enable, EEPROM beim Loeschen nicht mitloeschen, 
;Sytemtakt nicht durch 8 teilen, Low-Power Quarz-Oszillator.
;
;Es wird ein schwarzer Wii Nunchuk ausgelesen und seine Werte ueber die USART
;( RS232 spaeter mittels MAX3232 jetzt MAX232 ) an ein Terminalprogramm ge-
;sendet ( Hterm ). Bei meinem TTL <-> RS232-Wandler ist es noetig dort 
;DTR & RTS einzuschalten, da ueber eine "Trickschaltung" der MAX232 hierrueber
;mit Spannung versorgt wird, welche vorher zu messen ist, bevor der MAX daran
;angeschlossen wird.
;
;Ein einmaliger Tastendruck an PD7 ist noetig, um die Uebertragung zu starten.
;
;Hterm:
;1. Spalte = Joystick X-Achse
;2. Spalte = Joystick Y-Achse
;3. Spalte = X-Achse Beschleunigungssensor
;4. Spalte = Y-Achse Beschleunigungssensor
;5. Spalte = Z-Achse Beschleunigungssensor
;6. Spalte = Der Rest wie C-Button, Z-Button usw.
;
;Die Programmlaufanzeige (PLA) ist nicht so wie gedacht, da OCR0A = TOP ist und 
;dadurch der Timer/Counter0 Overflow Interrupt nicht dann kommt wie ich es
;urspruenglich vorgesehen hatte. 
;Auch die TWI-Schrittueberpruefung ist auskommentiert ( TWI_Status bzw. -
; Rueckmeldungen ), sowie das Auslesen der 16 Kalibrationsbytes.
;
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
; Einbinden der Controllerspezifischen Definitionsdatei
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
;
.NOLIST                 ;List-Output ausschalten
.INCLUDE "m88PAdef.inc" ;AVR-Definitionsdatei einbinden
.LIST                   ;List-Output wieder einschalten
;
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
; Arbeitsregister umbenennen
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;
;.DEF   = r0   ;
;.DEF   = r1   ;
;.DEF   = r2   ;
;.DEF   = r3   ;
;.DEF   = r4   ;
;.DEF   = r5   ;
;.DEF   = r6   ;
.DEF twi_data          = r7    ;TWI-Datenbyte 
.DEF null              = r8    ;Register mit dem Inhalt Null
;.DEF   = r9   ;
;.DEF   = r10	;
.DEF ovf0_zaehler      = r11    ;Timer0 Overflow-Zaehler
;.DEF   = r12   ;
.DEF twi_status_check  = r13    ;Vergleichsregister f. den Status-Check
.DEF zaehler           = r14    ;Allgemeiner Zaehler fuer verschiedene Sachen 
.DEF s_sreg            = r15    ;Zum Sichern des Statusregisters ( Nur 1 Takt )

.DEF ia                = r16    ;Interrupt Arbeitsregister ia ( Nur in ISR genutzt )
.DEF ib                = r17    ;Interrupt Arbeitsregister ib ( Nur in ISR genutzt )
;.DEF   = r18  ;     
.DEF flag_reg          = r19    ;Allgemeines Flaggenregister;
.DEF twi_ablauf_nr     = r20    ;Zeiger auf die jeweilige TWI-Aktion
.DEF key_reg           = r21    ;Fuer die Tasterentprellung
.DEF twi_txd           = r22    ;TWI-Datensendezeiger
.DEF twi_rxd           = r23    ;TWI-Datenempfangszeiger  
.DEF a                 = r24    ;Register 24 und 25 kann als universelles..
.DEF b                 = r25    ;..Doppelregister dienen
;.DEF xl               = r26    ;Der X-Zeiger ist die TWI-Statusnummertabellen-.. 
;.DEF xh               = r27    ;..adresse ( twi_status_nr_tab: )
;.DEF yl= r28   ;
;.DEF yh= r29   ;
;.DEF zl             = r30   ;Verwendung als Z-Zeiger, also wie bereits in der..
;.DEF zh             = r31   ;..Definitionsdatei von ATMEL angegeben
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
; Konstanten als Symbole
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
;Timer/Counter0
;
.EQU #ovf0_zaehler  = 4		;Wert f. Timer0 Overflowzaehler

.EQU #tc0_max       = $FF   ;Maximaler Zhlumfang ( Auflsung )

.EQU NINV0A         = 1<<COM0A1|0<<COM0A0;OC0A (Non-Inverting Mode)
.EQU INV0A          = 1<<COM0A1|1<<COM0A0;OC0A (Interting Mode)
.EQU TOGG0A         = 0<<COM0A1|1<<COM0A0;OC0A (Toggle Mode nur OC0A moeglich)

.EQU NINV0B         = 1<<COM0B1|0<<COM0B0;OC0B (Non-Inverting Mode)
.EQU INV0B          = 1<<COM0B1|1<<COM0B0;OC0B (Interting Mode)

.EQU Z_UMFANG       = OCR0A ;Zhlumfang
.EQU TASTGRAD       = OCR0B ;Tastgrad
;
;ADC
;
.EQU POTI_Z         = ~(1<<MUX3|1<<MUX2|1<<MUX1|1<<MUX0);Poti Zhlumfang ADC 0
.EQU POTI_T         = MUX1  ;Poti Tastgrad auf ADC-Kanal 1
;.EQU #ovf0_zaehler	    = 4		;Wert f. Timer0 Overflowzaehler

.EQU #ubrr			    = 2		;Wert f. USART Baudrate 38400 Baud

 
;
;TWI Konstanten
; 
.EQU #twi_sla_w         = $A4   ;Adresse des SLAVE ( Write )
.EQU #twi_sla_r         = $A5   ;Adresse des SLAVE ( Read  )

.EQU #twbr			    =  2	;TWI SCL-Frequenz(ca.102 kHz TWBR=1,2 ca.92kHz)
.EQU #twi_status_anz    = 25    ;Anzahl der Eintraege in der TWI-Statustabelle
.EQU #twi_ablauf_nr_anz = 31;20    ;Anzahl der einzelnen TWI-Aktionen
.EQU #twi_rxd_anz       =  6+16    ;Anzahl der einzulesenden Wii Nunchuk Daten
.EQU #twi_pos_anz       =  6    ;Anzahl der Positionsbytes
.EQU #twi_txd_anz       =  5;+2   ;Anzahl der sendenden Wii Nunchuk Daten

.EQU #twi_txd_end       = 17;9;8    ;Senden der Daten beenden
.EQU #twi_data_end      = 17    ;TWI-Ablaufnummer ab der Daten lesen endet.

.EQU #cmd_twi_start		= 1<<TWINT|1<<TWSTA|1<<TWEN|1<<TWIE ;TWI-START  Komando
.EQU #cmd_twi_stop      = 1<<TWINT|1<<TWSTO|1<<TWEN|1<<TWIE ;TWI-STOP   Komando
.EQU #cmd_twi_ack       = 1<<TWINT|1<<TWEA |1<<TWEN|1<<TWIE ;TWI-ACKNOWLEDGE 
.EQU #cmd_twi_action    = 1<<TWINT         |1<<TWEN|1<<TWIE ;TWI-Aktion 
;
;
;
;Bitbezeichner fuer das allgemeine Flaggenregister flag_reg
;
;.equ			= 0	;
;.equ			= 1	;
;.equ			= 2	;
;.equ  	        = 3;
.equ	pla.flg		= 4	;Programmlaufanzeige-Flagge
.equ	twi.err 	= 5	;TWI-ERROR-Flagge
;.equ		    = 6	;
;.equ		    = 7	; 
;
;Bitbezeichner fuer das Tasterentprellregister key_reg
;
;.equ			= 0	;Taster..
;.equ			= 1	;..Prellzaehler
.equ  key.unactuated	= 2	;Taster entprellt unbetaetigt
.equ  key.actuated		= 3	;Taster entprellt betaetigt
;.equ			= 4	;
;.equ	        = 5	;
;.equ	        = 6	; 
;.equ		    = 7	; 